
//
//	Richard P.
//	Nov 2011
//
// --------------------------------------------------------------------
//	take LVDS input, delay and input ddr
//	format to 8 channels of sample data out to dumpmem
//  


module adcif(
   // PAR LVDS interface
input                     clk_adc_ext,
input     [28:0]          din,

// config interface
input	[1:0]		cap_depth,
input	[2:0]		cap_format,
input	[7:0]		cap_chans,
input				inv_data,
input				inv_clk,
input				lsb_first,

   // output ADC parallel word data
output [15:0]  		dout1,
output [15:0]  		dout2,
output [15:0]  		dout3,
output [15:0]  		dout4,
output [15:0]  		dout5,
output [15:0] 		dout6,
output [15:0]  		dout7,
output [15:0]  		dout8,
output              clk_adc,
input               reset_n
   );


wire                   	adc_sen;
reg                    	adc_sclk;
reg                    	adc_sdata;
wire                   	adc_cfg1;
wire                   	adc_cfg2;
wire                   	adc_cfg3;
wire                   	adc_cfg4;
wire                   	clk2;
wire					clkintfb;
wire					clkok;
wire					clkos;

wire 	[28:0]			delay_din;
wire 	[28:0]			dinA0;         // A0 bits from IDDR2x
wire 	[28:0]			dinB0;         // B0 bits from IDDR2x
wire 	[28:0]			dinA1;         // A1 bits from IDDR2x
wire 	[28:0]			dinB1;         // B1 bits from IDDR2x

wire      				rst;

reg		[15:0]		cntr;
reg		[15:0]		cntr_r;
reg		[15:0]		format_dout1,format_dout2,format_dout3,format_dout4,format_dout5,format_dout6,format_dout7,format_dout8;

assign rst = ~reset_n;

always @ (posedge clk_adc_ext)
	if (rst)
			cntr <= 16'h0000;
	else
		begin
			cntr <= cntr +1;
			cntr_r <= cntr;
		end
			
always @(posedge clkok)
begin	// 1ch bit-wise DDR   store 2 samples/clock
	format_dout1	<= {~dinB0[12],~dinA0[12],~dinB0[11],~dinA0[11],~dinB0[10],~dinA0[10],~dinB0[9],~dinA0[9],~dinB0[7],~dinA0[7],~dinB0[6],~dinA0[6],~dinB0[5],~dinA0[5],~dinB0[4],~dinA0[4]};
	format_dout2	<= {~dinB0[12],~dinA0[12],~dinB0[11],~dinA0[11],~dinB0[10],~dinA0[10],~dinB0[9],~dinA0[9],~dinB0[7],~dinA0[7],~dinB0[6],~dinA0[6],~dinB0[5],~dinA0[5],~dinB0[4],~dinA0[4]};
	format_dout3	<= {~dinB0[12],~dinA0[12],~dinB0[11],~dinA0[11],~dinB0[10],~dinA0[10],~dinB0[9],~dinA0[9],~dinB0[7],~dinA0[7],~dinB0[6],~dinA0[6],~dinB0[5],~dinA0[5],~dinB0[4],~dinA0[4]};
	format_dout4	<= {~dinB0[12],~dinA0[12],~dinB0[11],~dinA0[11],~dinB0[10],~dinA0[10],~dinB0[9],~dinA0[9],~dinB0[7],~dinA0[7],~dinB0[6],~dinA0[6],~dinB0[5],~dinA0[5],~dinB0[4],~dinA0[4]};
	format_dout5	<= {~dinB1[12],~dinA1[12],~dinB1[11],~dinA1[11],~dinB1[10],~dinA1[10],~dinB1[9],~dinA1[9],~dinB1[7],~dinA1[7],~dinB1[6],~dinA1[6],~dinB1[5],~dinA1[5],~dinB1[4],~dinA1[4]};
	format_dout6	<= {~dinB1[12],~dinA1[12],~dinB1[11],~dinA1[11],~dinB1[10],~dinA1[10],~dinB1[9],~dinA1[9],~dinB1[7],~dinA1[7],~dinB1[6],~dinA1[6],~dinB1[5],~dinA1[5],~dinB1[4],~dinA1[4]};
	format_dout7	<= {~dinB1[12],~dinA1[12],~dinB1[11],~dinA1[11],~dinB1[10],~dinA1[10],~dinB1[9],~dinA1[9],~dinB1[7],~dinA1[7],~dinB1[6],~dinA1[6],~dinB1[5],~dinA1[5],~dinB1[4],~dinA1[4]};
	format_dout8	<= {~dinB1[12],~dinA1[12],~dinB1[11],~dinA1[11],~dinB1[10],~dinA1[10],~dinB1[9],~dinA1[9],~dinB1[7],~dinA1[7],~dinB1[6],~dinA1[6],~dinB1[5],~dinA1[5],~dinB1[4],~dinA1[4]};
end

			
assign		dout1	= format_dout1;
assign		dout2	= format_dout2;
assign		dout3	= format_dout3;
assign		dout4	= format_dout4;
assign		dout5	= format_dout5;
assign		dout6	= format_dout6;
assign		dout7	= format_dout7;
assign		dout8	= format_dout8;
assign 		clk_adc	=	clkok;

CLKDIVB div2 (
   .CLKI   (clk_adc_ext),
   .RST    (rst),
   .RELEASE(1'b1),
   .CDIV1  (),
   .CDIV2  (clkok),
   .CDIV4  (),
   .CDIV8  ());
   
//	defparam Inst3_EHXPLLF.FEEDBK_PATH = "INTERNAL" ;
//    defparam Inst3_EHXPLLF.CLKOK_BYPASS = "DISABLED" ;
//    defparam Inst3_EHXPLLF.CLKOS_BYPASS = "DISABLED" ;
//    defparam Inst3_EHXPLLF.CLKOP_BYPASS = "DISABLED" ;
//    defparam Inst3_EHXPLLF.CLKOK_INPUT = "CLKOP" ;
//    defparam Inst3_EHXPLLF.DELAY_PWD = "DISABLED" ;
//    defparam Inst3_EHXPLLF.DELAY_VAL = 0 ;
//    defparam Inst3_EHXPLLF.CLKOS_TRIM_DELAY = 0 ;
//    defparam Inst3_EHXPLLF.CLKOS_TRIM_POL = "RISING" ;
//    defparam Inst3_EHXPLLF.CLKOP_TRIM_DELAY = 0 ;
//    defparam Inst3_EHXPLLF.CLKOP_TRIM_POL = "RISING" ;
//    defparam Inst3_EHXPLLF.PHASE_DELAY_CNTL = "DYNAMIC" ;
//    defparam Inst3_EHXPLLF.DUTY = 8 ;
//    defparam Inst3_EHXPLLF.PHASEADJ = "0.0" ;
//    defparam Inst3_EHXPLLF.CLKOK_DIV = 2 ;
//    defparam Inst3_EHXPLLF.CLKOP_DIV = 2 ;
//    defparam Inst3_EHXPLLF.CLKFB_DIV = 1 ;
//    defparam Inst3_EHXPLLF.CLKI_DIV = 1 ;
//    defparam Inst3_EHXPLLF.FIN = "275.000000" ;
//    EHXPLLF Inst3_EHXPLLF (.CLKI(clk_adc_ext), .CLKFB(clkintfb), .RST(rst), 
//        .RSTK(1'b0), .WRDEL(1'b0), 
//		.DRPAI3(cap_chans[3]), .DRPAI2(cap_chans[2]), .DRPAI1(cap_chans[1]), .DRPAI0(cap_chans[0]), 
//		.DFPAI3(~cap_chans[3]), .DFPAI2(cap_chans[2]), .DFPAI1(cap_chans[1]), .DFPAI0(cap_chans[0]), 
//        .FDA3(1'b0), .FDA2(1'b0), .FDA1(1'b0), .FDA0(1'b0), 
//        .CLKOP(), .CLKOS(clkos), .CLKOK(clkok), .CLKOK2(), .LOCK(), 
//        .CLKINTFB(clkintfb))
             /* synthesis FREQUENCY_PIN_CLKOS="275.000000" */
             /* synthesis FREQUENCY_PIN_CLKOP="275.000000" */
             /* synthesis FREQUENCY_PIN_CLKI="275.000000" */
             /* synthesis FREQUENCY_PIN_CLKOK="137.500000" */

//    assign eclk = clkos;
//    assign sclk_t = clkok;


   
DELAYB delay0[28:0] (
   .A(din),
   .DEL0(1'b0),
   .DEL1(1'b0),
   .DEL2(1'b0),
   .DEL3(1'b0),
   .Z(delay_din));
      
 IDDRX2D1 Inst1_IDDRX2D0[28:0] (
       .D(delay_din),
       .SCLK(clkok),
       .ECLK(clk_adc_ext),
       .QA0(dinA0),
       .QB0(dinB0),
       .QA1(dinA1),
       .QB1(dinB1))/*synthesis IDDRAPPS = "PLL_DYNAMIC" */;

// now - formatting - assign bits to doutn
// set delay taps

endmodule
