//============================================================================
//
//This confidential and proprietary software may be used only as authorized
//by a licensing agreement from Texas Instruments, US.
//
//     In the event of publication, the following notice is applicable:
//
//                        (C) COPYRIGHT 2009 TI
//                          ALL RIGHTS RESERVED
//
// Entire notice above must be reproduced on all authorized copies.
//
//
// Title:      %M%
// Version:    %I%
// Date:       %G%
// Author:     Hamza Fraz, edited to output 8 banks of FIFO by Richard Prentice
//============================================================================


module dumpmem_mspi(

//	this is a spi-slave, configured for CPHA = 1 and CPOL = 1 according to
//  http://en.wikipedia.org/wiki/File:SPI_timing_diagram.svg
//	8-bit data length
//	LSB first

input						clk_sys,
input						reset_n,
input						clk_spi,
input		[7:0]			dvalid,
input		[(8*16 -1):0]	data,
input						spi_ss,

output	reg					oe_fifo_r,
output	reg					miso_r
);

//-----------------------------------------------------------------------------

reg				clkspi_negedge_r;
reg				clkspi_r;
reg				clkspi_d1_r;
reg		[3:0]	dvalid_r;
reg		[2:0]	bitcntr_r;
reg		[1:0]	bytecntr_r;
reg		[16:0]	spiframe_r;
reg		[15:0]	din;
reg		[15:0]	din76;
reg		[15:0]	din54;
reg		[15:0]	din32;
reg		[15:0]	din10;
reg		[15:0]	din76_r;
reg		[15:0]	din54_r;
reg		[15:0]	din32_r;
reg		[15:0]	din10_r;
reg				dvalid_b;

//-----------------------------------------------------------------------------

wire	[16:0]	spiframe_c;
wire	[16:0]	spiframe_i;
wire			clkspi_negedge;
wire			bitcntr_nz;
wire			bitcntr_z;
wire			bytecntr_nz;
wire			bytecntr_z;
wire			oe_fifo;
wire			dvalid32;
wire			dvalid10;
wire			bitcntr_az;

//-----------------------------------------------------------------------------
// sync SPI Clk to system clock, make single pulse off of SPI Clk negedge

always @(posedge clk_sys)
begin
	clkspi_r	<= clk_spi;
	clkspi_d1_r	<= clkspi_r;

	clkspi_negedge_r	<= clkspi_negedge;
end

assign	clkspi_negedge = clkspi_r & ~clkspi_d1_r;  // trying posedge

//-----------------------------------------------------------------------------
// 8 bit counter to count our way through shifting out a byte

always @(posedge clk_sys or negedge reset_n)
begin
	if(!reset_n)
		bitcntr_r	<= 0;
	else if(clkspi_negedge && bitcntr_z && ~spi_ss)
		bitcntr_r	<= 3'd7;
	else if(clkspi_negedge && ~spi_ss)
		bitcntr_r	<= bitcntr_r - 1'b1;
end

assign	bitcntr_nz	= |bitcntr_r;
assign	bitcntr_z	= ~bitcntr_nz;
assign 	bitcntr_az	= ~bitcntr_r[2] & ~bitcntr_r[1] & bitcntr_r[0];

//-----------------------------------------------------------------------------
// byte counter to count out bytes in a sample.  2 bytes/sample.  more of a toggle than counter

always @(posedge clk_sys or negedge reset_n)
begin
	if(!reset_n)
		bytecntr_r	<= 0;
	else if(clkspi_negedge && bitcntr_z && bytecntr_z && ~spi_ss)
		bytecntr_r	<= 1'b1;
	else if(clkspi_negedge && bitcntr_z && bytecntr_nz && ~spi_ss)
		bytecntr_r	<= bytecntr_r - 1'b1;
end

assign	bytecntr_nz	= |bytecntr_r;
assign	bytecntr_z	= ~bytecntr_nz;

//-----------------------------------------------------------------------------
// make output enable pulse to output one more sample from FIFO bank

assign	oe_fifo = clkspi_negedge & bytecntr_z & bitcntr_az;

always @(posedge clk_sys)
begin
	oe_fifo_r <= oe_fifo;
end

//-----------------------------------------------------------------------------
// assuming only one bank of FIFO at a time gets an enable and thus only one bank valid
// cases broken down into 2 by 2 until data from the one valid bank gets assinged
// a lot of code to make a mux, IMHO.  rmp

always @(*)
begin
	case(dvalid[7:6])
		2'b10	:	din76 = data[(8*16 - 1)	:	7*16];
		2'b01	:	din76 = data[(7*16 - 1)	:	6*16];
		default	:	din76 = data[(7*16 - 1)	:	6*16];
	endcase
end

always @(*)
begin
	case(dvalid[5:4])
		2'b10	:	din54 = data[(6*16 - 1)	:	5*16];
		2'b01	:	din54 = data[(5*16 - 1)	:	4*16];
		default	:	din54 = data[(4*16 - 1)	:	3*16];
	endcase
end
always @(*)
begin
	case(dvalid[3:2])
		2'b10	:	din32 = data[(4*16 - 1)	:	3*16];
		2'b01	:	din32 = data[(3*16 - 1)	:	2*16];
		default	:	din32 = data[(3*16 - 1)	:	2*16];
	endcase
end

always @(*)
begin
	case(dvalid[1:0])
		2'b10	:	din10 = data[(2*16 - 1)	:	16];
		2'b01	:	din10 = data[(16 - 1)	:	0];
		default	:	din10 = data[(16 - 1)	:	0];
	endcase
end

assign	dvalid76 = |dvalid[7:6];
assign	dvalid54 = |dvalid[5:4];
assign	dvalid32 = |dvalid[3:2];
assign	dvalid10 = |dvalid[1:0];

//-----------------------------------------------------------------------------

always @(posedge clk_sys)
begin
	if(dvalid76)
		din76_r		<= din76;

	if(dvalid54)
		din54_r		<= din54;

	if(dvalid32)
		din32_r		<= din32;

	if(dvalid10)
		din10_r		<= din10;
		
	dvalid_b		<=	(dvalid76 || dvalid54  || dvalid32 || dvalid10);
end

always @(posedge clk_sys or negedge reset_n)
begin
	if(!reset_n)
		dvalid_r	<= 0;
	else if(dvalid76 || dvalid54  || dvalid32 || dvalid10)
		dvalid_r	<= {dvalid76,dvalid54,dvalid32,dvalid10};
end

//-----------------------------------------------------------------------------

always @(*)
begin
	case(dvalid_r)
		4'b1000	:	din = din76_r;
		4'b0100	:	din = din54_r;
		4'b0010	:	din = din32_r;
		4'b0001	:	din = din10_r;
		default	:	din = din10_r;  
	endcase
end

//-----------------------------------------------------------------------------
// on that one OE FIFO pulse chunk the incoming data into a register
// why not reduce the code?  rmp

always @(posedge clk_sys)
begin
	if(dvalid_b)
		spiframe_r[3:0]	<= spiframe_i[3:0];
	else if(clkspi_negedge)
		spiframe_r[3:0]	<= spiframe_c[3:0];

	if(dvalid_b)
		spiframe_r[7:4]	<= spiframe_i[7:4];
	else if(clkspi_negedge)
		spiframe_r[7:4]	<= spiframe_c[7:4];

	if(dvalid_b)
		spiframe_r[11:8]	<= spiframe_i[11:8];
	else if(clkspi_negedge)
		spiframe_r[11:8]	<= spiframe_c[11:8];

	if(dvalid_b)
		spiframe_r[16:12]	<= spiframe_i[16:12];
	else if(clkspi_negedge)
		spiframe_r[16:12]	<= spiframe_c[16:12];

end

assign	spiframe_i	= {din,1'b0};
assign	spiframe_c	= {spiframe_r[0],spiframe_r[16:1]};

//-----------------------------------------------------------------------------
//  Finally, the shift out

always @(posedge clk_sys)
	miso_r <= spiframe_c[0];

//-----------------------------------------------------------------------------

endmodule
