//============================================================================
//
//This confidential and proprietary software may be used only as authorized
//by a licensing agreement from Texas Instruments, US.
//
//     In the event of publication, the following notice is applicable:
//
//                        (C) COPYRIGHT 2009 TI
//                          ALL RIGHTS RESERVED
//
// Entire notice above must be reproduced on all authorized copies.
//
//
// Title:      %M%
// Version:    %I%
// Date:       %G%
// Author:     Hamza Fraz
//============================================================================



module dumpmem_wctrl(
input				clk,

input			[3:0]	mode,
input			[7:0]	afull_fifo,
input				wstart_p,

output	reg	[7:0]	bew_fifo
);

//-----------------------------------------------------------------------------

reg		[3:0]	winc_fifo;
reg		[3:0]	winc_fifo_r;
reg		[7:0]	be_fifo_m0_r;
reg		[7:0]	be_fifo_m1_r;
reg		[7:0]	be_fifo_m2_r;
reg		[7:0]	be_fifo_m3_r;
reg		[7:0]	be_fifo_m0;
reg		[7:0]	be_fifo_m1;
reg		[7:0]	be_fifo_m2;
reg		[7:0]	be_fifo_m3;
reg		[7:0]	afull_fifo_r;
reg		[7:0]	afull_fifo_pr;
reg				wstart_pr;

//-----------------------------------------------------------------------------

wire	[7:0]	afull_fifo_p;

//-----------------------------------------------------------------------------

always @(posedge clk)
begin
	wstart_pr <= wstart_p;
	if(wstart_pr)
	begin
		be_fifo_m0_r <= 8'b0000_0001;
		be_fifo_m1_r <= 8'b0001_0001;
		be_fifo_m2_r <= 8'b0101_0101;
		be_fifo_m3_r <= 8'b1111_1111;
	end
	else
	begin
		if(winc_fifo_r[0])
			be_fifo_m0_r <= be_fifo_m0;

		if(winc_fifo_r[1])
			be_fifo_m1_r <= be_fifo_m1;

		if(winc_fifo_r[2])
			be_fifo_m2_r <= be_fifo_m2;

		if(winc_fifo_r[3])
			be_fifo_m3_r <= be_fifo_m3;
	end
end

//-----------------------------------------------------------------------------

always @(*)
begin
	be_fifo_m0 = be_fifo_m0_r << 1;

	be_fifo_m1[7:4] = be_fifo_m1_r[7:4] << 1;
	be_fifo_m1[3:0] = be_fifo_m1_r[3:0] << 1;

	be_fifo_m2[7:6] = be_fifo_m2_r[7:6] << 1;
	be_fifo_m2[5:4] = be_fifo_m2_r[5:4] << 1;
	be_fifo_m2[3:2] = be_fifo_m2_r[3:2] << 1;
	be_fifo_m2[1:0] = be_fifo_m2_r[1:0] << 1;

	be_fifo_m3 = 8'b0;
end

//-----------------------------------------------------------------------------

always @(posedge clk)
begin
	afull_fifo_r	<= afull_fifo;
	afull_fifo_pr	<= afull_fifo_p;
end

assign	afull_fifo_p = afull_fifo & ~afull_fifo_r;

always @(afull_fifo_pr)
begin
	winc_fifo[0] = |afull_fifo_pr[7:0];
	winc_fifo[1] = |afull_fifo_pr[7:4];
	winc_fifo[2] = |afull_fifo_pr[7:6];
	winc_fifo[3] =  afull_fifo_pr[7];
end

//-----------------------------------------------------------------------------

always @(posedge clk)
begin
	winc_fifo_r <= winc_fifo;
end

//-----------------------------------------------------------------------------

always @(*)
begin
	case(mode)
		4'b1000	:	bew_fifo = be_fifo_m3_r;
		4'b0100	:	bew_fifo = be_fifo_m2_r;
		4'b0010	:	bew_fifo = be_fifo_m1_r;
		4'b0001	:	bew_fifo = be_fifo_m0_r;
		default	:	bew_fifo = be_fifo_m0_r;
	endcase
end

//-----------------------------------------------------------------------------

endmodule
