
`timescale 1ns/ 10 ps

module TSW1405_1ch_bit_wise(
	reset_n,
	clk_lvds_rx_p,
	clk_spi,
	spi_mosi,
	spi_ss,
//	jumpers,
	lvds_rx_port0_p,
	lvds_rx_port1_p,
	spi_miso,
	led, 
	regbit 
	);

input			reset_n;		//pin C21

input			clk_lvds_rx_p;	//pin L4
input			clk_spi;		//pin A20

input	[16:0]	lvds_rx_port0_p;	//
input	[11:0]	lvds_rx_port1_p;	//

input	 		spi_mosi;		//pin A21
input	 		spi_ss;			//pin B20

//input	[3:0]	jumpers;		//pins G18, G19, G20, H17
//input			ext_sync;

output			spi_miso;		//pin B19
output 			led;
output			regbit;

   
	wire			clk_osc;
	wire 			led;
	wire			regbit;
	wire			sys_rst_n;
	reg		[27:0] 	counter;
	reg				slow_clk;
    reg				wren;
	
	wire	[15:0]	din_fifo1;	// 8-channel ADC data
	wire	[15:0]	din_fifo2;
	wire	[15:0]	din_fifo3;
	wire	[15:0]	din_fifo4;
	wire	[15:0]	din_fifo5;
	wire	[15:0]	din_fifo6;
	wire	[15:0]	din_fifo7;
	wire	[15:0]	din_fifo8;
	wire	[15:0]	dout_fifo;
	wire			clk_adc;

	wire	[1:0]	cap_depth;
	wire	[2:0]	cap_format;
	wire	[7:0]	cap_chans;
	wire			inv_data;
	wire			inv_clk;
	wire			lsb_first;
	wire			SW_reset;


//-----------------------------------------------------------------------------

	reg				capture_sync1_r;
	reg				capture_sync2_r;
	reg		[25:0]	testcnt_r;

	
OSCF OSCInst0 (.OSC(clk_osc));
defparam OSCInst0.NOM_FREQ = "130.0" ;

tsw1405_reset reset_sys_inst(
				.rst_n		(sys_rst_n),
				.clk		(clk_osc),
				.sw_reset	(SW_reset),
				.reset_n	(reset_n)
				);

tsw1405_reset reset_adcif_inst(
				.rst_n		(adc_rst_n),
				.clk		(clk_lvds_rx_p),
				.sw_reset	(SW_reset),
				.reset_n	(reset_n)
				);


always @(posedge clk_adc)
   if (counter == 28'h2faf078)
	  begin
	  counter   <=  28'd0;
	  slow_clk  <=  ~slow_clk;
	  end
	else
	  counter <=  counter + 1;
	  
assign  led  = slow_clk;

assign 	regbit = lsb_first;

adcif adcif_inst(
		.reset_n		(adc_rst_n),			//
		.din			({lvds_rx_port1_p,lvds_rx_port0_p}),		//
		.clk_adc_ext	(clk_lvds_rx_p),						//
		
		.cap_depth		(cap_depth),
		.cap_format		(cap_format),
		.cap_chans		(cap_chans),
		.inv_data		(inv_data),
		.inv_clk		(inv_clk),
		.lsb_first		(lsb_first),

		.clk_adc		(clk_adc),				//
		.dout1			(din_fifo1),			//
		.dout2			(din_fifo2),			//
		.dout3			(din_fifo3),			//
		.dout4			(din_fifo4),			//
		.dout5			(din_fifo5),			//
		.dout6			(din_fifo6),			//
		.dout7			(din_fifo7),			//
		.dout8			(din_fifo8)			//
		);




dumpmem_top	dumpmem_top_inst(
		.clk_adc		(clk_adc),
		.clk_sys		(clk_osc),
		.clk_spi		(clk_spi),
		.spi_ss			(spi_ss),
		.reset_n		(sys_rst_n), 
		.din_fifo1		(din_fifo1),
		.din_fifo2		(din_fifo2),
		.din_fifo3		(din_fifo3),
		.din_fifo4		(din_fifo4),
		.din_fifo5		(din_fifo5),
		.din_fifo6		(din_fifo6),
		.din_fifo7		(din_fifo7),
		.din_fifo8		(din_fifo8),
		
		.cap_depth		(cap_depth),
		.cap_format		(cap_format),
		.cap_chans		(cap_chans),
		.inv_data		(inv_data),
		.inv_clk		(inv_clk),
		.lsb_first		(lsb_first),
		.SW_reset		(SW_reset),

		.mosi			(spi_mosi),
		.miso			(spi_miso)
	);


		
   
   
endmodule