//============================================================================
//
//This confidential and proprietary software may be used only as authorized
//by a licensing agreement from Texas Instruments, US.
//
//     In the event of publication, the following notice is applicable:
//
//                        (C) COPYRIGHT 2009 TI
//                          ALL RIGHTS RESERVED
//
// Entire notice above must be reproduced on all authorized copies.
//
//
// Title:      %M%
// Version:    %I%
// Date:       %G%
// Author:     Hamza Fraz, edited for TSW1405 Richard Prentice
//============================================================================

module dumpmem(
input			clk_adc,
input			clk_sys,
input			clk_spi,
input			reset_n,

	// following signals are synchonous to clk_adc
input	[15:0]	din_fifo1,	
input	[15:0]	din_fifo2,	
input	[15:0]	din_fifo3,	
input	[15:0]	din_fifo4,	
input	[15:0]	din_fifo5,	
input	[15:0]	din_fifo6,	
input	[15:0]	din_fifo7,	
input	[15:0]	din_fifo8,	

	// following signals are synchonous to clk_sys
input	[1:0]	cap_depth,
input	[2:0]	cap_format,
input	[7:0]	cap_chans,
input			inv_data,
input			inv_clk,
input			lsb_first,
input	[7:0]	flush_fifo,
input			we_fifo,
input	[3:0]	mode,
output			done,
output	[15:0]	dout_fifo1,



	// following signals are synchonous to respective spi clocks
input		spi_ss,
output		mem_miso
	);

//-----------------------------------------------------------------------------

reg		[7:0]	oe_fifo_d1_r;
reg		[7:0]	oe_fifo_d1_r2;
reg		[7:0]	valid_fifo_r;
reg				we_fifo_sync0_r;
reg				wstart_r;
reg				wstart_d1_r;
reg				rstart_r;
reg				rstart_d1_r;
reg				empty_fifo_r;
reg				empty_fifo_r2;
reg				interrupt_d1_r;
reg				interrupt_r;
reg				done;
reg				wefifo_r;

//-----------------------------------------------------------------------------

// wire	[15:0]	dout_fifo1;
wire	[15:0]	dout_fifo2;
wire	[15:0]	dout_fifo3;
wire	[15:0]	dout_fifo4;
wire	[15:0]	dout_fifo5;
wire	[15:0]	dout_fifo6;
wire	[15:0]	dout_fifo7;
wire	[15:0]	dout_fifo8;
wire	[7:0]	full_fifo;
wire	[7:0]	afull_fifo;
wire	[7:0]	empty_fifo;
wire	[7:0]	aempty_fifo;
wire	[7:0]	wren_fifo;
wire	[7:0]	rden_fifo;
wire	[7:0]	bew_fifo;
wire	[7:0]	ber_fifo;
wire			oe_fifo;
wire			wstart_p;
wire			rstart_p;
wire			rbz_fifo;
wire			interrupt_p;

//-----------------------------------------------------------------------------

assign	wren_fifo = bew_fifo & {8{wefifo_r}};

assign	rden_fifo[7:0] = (ber_fifo[7:0] & {8{oe_fifo}}) | {7'b0,interrupt_p};


//-----------------------------------------------------------------------------

dumpmem_dp dumpmem_dp_inst0(
			.Data			(din_fifo1),
			.WrClock		(clk_adc),
			.RdClock		(clk_sys),
			.WrEn			(wren_fifo[0]),
			.RdEn			(rden_fifo[0]),
			.Reset			(~reset_n),
			.RPReset		(flush_fifo[0]),
			.Q				(dout_fifo1),
			.Empty			(empty_fifo[0]),
			.Full			(full_fifo[0]),
			.AlmostEmpty	(aempty_fifo[0]),
			.AlmostFull		(afull_fifo[0])
			);

dumpmem_dp dumpmem_dp_inst1(
			.Data			(din_fifo2),
			.WrClock		(clk_adc),
			.RdClock		(clk_sys),
			.WrEn			(wren_fifo[1]),
			.RdEn			(rden_fifo[1]),
			.Reset			(~reset_n),
			.RPReset		(flush_fifo[1]),
			.Q				(dout_fifo2),
			.Empty			(empty_fifo[1]),
			.Full			(full_fifo[1]),
			.AlmostEmpty	(aempty_fifo[1]),
			.AlmostFull		(afull_fifo[1])
			);

dumpmem_dp dumpmem_dp_inst2(
			.Data			(din_fifo3),
			.WrClock		(clk_adc),
			.RdClock		(clk_sys),
			.WrEn			(wren_fifo[2]),
			.RdEn			(rden_fifo[2]),
			.Reset			(~reset_n),
			.RPReset		(flush_fifo[2]),
			.Q				(dout_fifo3),
			.Empty			(empty_fifo[2]),
			.Full			(full_fifo[2]),
			.AlmostEmpty	(aempty_fifo[2]),
			.AlmostFull		(afull_fifo[2])
			);

dumpmem_dp dumpmem_dp_inst3(
			.Data			(din_fifo4),
			.WrClock		(clk_adc),
			.RdClock		(clk_sys),
			.WrEn			(wren_fifo[3]),
			.RdEn			(rden_fifo[3]),
			.Reset			(~reset_n),
			.RPReset		(flush_fifo[3]),
			.Q				(dout_fifo4),
			.Empty			(empty_fifo[3]),
			.Full			(full_fifo[3]),
			.AlmostEmpty	(aempty_fifo[3]),
			.AlmostFull		(afull_fifo[3])
			);

dumpmem_dp dumpmem_dp_inst4(
			.Data			(din_fifo5),
			.WrClock		(clk_adc),
			.RdClock		(clk_sys),
			.WrEn			(wren_fifo[4]),
			.RdEn			(rden_fifo[4]),
			.Reset			(~reset_n),
			.RPReset		(flush_fifo[4]),
			.Q				(dout_fifo5),
			.Empty			(empty_fifo[4]),
			.Full			(full_fifo[4]),
			.AlmostEmpty	(aempty_fifo[4]),
			.AlmostFull		(afull_fifo[4])
			);

dumpmem_dp dumpmem_dp_inst5(
			.Data			(din_fifo6),
			.WrClock		(clk_adc),
			.RdClock		(clk_sys),
			.WrEn			(wren_fifo[5]),
			.RdEn			(rden_fifo[5]),
			.Reset			(~reset_n),
			.RPReset		(flush_fifo[5]),
			.Q				(dout_fifo6),
			.Empty			(empty_fifo[5]),
			.Full			(full_fifo[5]),
			.AlmostEmpty	(aempty_fifo[5]),
			.AlmostFull		(afull_fifo[5])
			);

dumpmem_dp dumpmem_dp_inst6(
			.Data			(din_fifo7),
			.WrClock		(clk_adc),
			.RdClock		(clk_sys),
			.WrEn			(wren_fifo[6]),
			.RdEn			(rden_fifo[6]),
			.Reset			(~reset_n),
			.RPReset		(flush_fifo[6]),
			.Q				(dout_fifo7),
			.Empty			(empty_fifo[6]),
			.Full			(full_fifo[6]),
			.AlmostEmpty	(aempty_fifo[6]),
			.AlmostFull		(afull_fifo[6])
			);

dumpmem_dp dumpmem_dp_inst7(
			.Data			(din_fifo8),
			.WrClock		(clk_adc),
			.RdClock		(clk_sys),
			.WrEn			(wren_fifo[7]),
			.RdEn			(rden_fifo[7]),
			.Reset			(~reset_n),
			.RPReset		(flush_fifo[7]),
			.Q				(dout_fifo8),
			.Empty			(empty_fifo[7]),
			.Full			(full_fifo[7]),
			.AlmostEmpty	(aempty_fifo[7]),
			.AlmostFull		(afull_fifo[7])
			);

//-----------------------------------------------------------------------------

always @(posedge clk_adc)
begin
	we_fifo_sync0_r	<= we_fifo;
	wstart_r		<= we_fifo_sync0_r;
	wstart_d1_r		<= wstart_r;
	wefifo_r		<= wstart_d1_r;
end

assign	wstart_p = wstart_r & ~wstart_d1_r;

//-----------------------------------------------------------------------------

always @(posedge clk_sys)
begin
	rstart_r	<= we_fifo;
	rstart_d1_r	<= rstart_r;
end

assign	rstart_p = rstart_r & ~rstart_d1_r;

//-----------------------------------------------------------------------------

always @(posedge clk_sys)
begin
	empty_fifo_r		<= &empty_fifo;
	empty_fifo_r2		<= empty_fifo_r;
	done				<= empty_fifo_r & ~empty_fifo_r2; // single pulse to clear capture bit when fifos go empty
	oe_fifo_d1_r		<= rden_fifo;
	oe_fifo_d1_r2		=	oe_fifo_d1_r;

	if(!empty_fifo_r) 
		valid_fifo_r	<= oe_fifo_d1_r2;  //  still not sure this lines up - later shouldn't hurt
	else
		valid_fifo_r	<= 8'b0;
end

//-----------------------------------------------------------------------------

dumpmem_wctrl dumpmem_wctrl_inst(
			.clk		(clk_adc),			// input

			.mode		(mode),				// input
			.afull_fifo	(afull_fifo),		// input
			.wstart_p	(wstart_p),			// input

			.bew_fifo	(bew_fifo)			// output
			);

//-----------------------------------------------------------------------------

dumpmem_rctrl dumpmem_rctrl_inst(
			.clk		(clk_sys),			// input

			.oe_fifo	(oe_fifo),			// input
			.aempty_fifo(aempty_fifo[7:0]),	// input
			.empty_fifo	(empty_fifo[7:0]),	// input
			.rstart_p	(rstart_p),			// input

			.ber_fifo	(ber_fifo[7:0])  		// output
			);

assign	rbz_fifo = |ber_fifo;

//-----------------------------------------------------------------------------

dumpmem_mspi dumpmem_mspi_inst(
			.clk_sys	(clk_sys),			// input
			.reset_n	(reset_n),			// input
			.clk_spi	(clk_spi),			// input
			.dvalid		(valid_fifo_r),		// input
			.data		({dout_fifo8,dout_fifo7,dout_fifo6,dout_fifo5,dout_fifo4,dout_fifo3,dout_fifo2,dout_fifo1}),
			.spi_ss		(spi_ss),			// input
			.oe_fifo_r	(oe_fifo),			// output
			.miso_r		(mem_miso)			// output
		);

//-----------------------------------------------------------------------------

always @(posedge clk_sys or negedge reset_n)
begin
	if(!reset_n)
		interrupt_r <= 1'b0;
	else
		interrupt_r <= &full_fifo;
end

always @(posedge clk_sys or negedge reset_n)
begin
	if(!reset_n)
		interrupt_d1_r <= 1'b0;
	else
		interrupt_d1_r <= interrupt_r;
end

assign	interrupt_p = interrupt_r & ~interrupt_d1_r;

//-----------------------------------------------------------------------------

//but - no muxing of what channel or channels to put in fifo
// no selection of how much fifo to fill
// no selection of what or how much to empty fifo

endmodule
