
`timescale 1ns/ 10 ps

module jtag_interface
  (
  input             rstn_i    , //  Global Active Low Reset
  input             sys_clk_i , //  Global FPGA System Clock
  input				done,
  input		[15:0]	dout_fifo1,
  output  reg [7:0] ctrl0_r      , //  Read Data from Register 0
  output  reg [7:0] ctrl1_r      , //  Read Data from Register 1
  output  reg [7:0] ctrl2_r      , //  Read Data from Register 2
  output  reg [7:0] ctrl3_r      , //  Read Data from Register 3
  output  reg [7:0] ctrl4_r      , //  Read Data from Register 4
  output  reg [7:0] ctrl5_r      , //  Read Data from Register 5
  output  reg [7:0] ctrl6_r      , //  Read Data from Register 6
  output  reg [7:0] ctrl7_r      , //  Read Data from Register 7
  output  reg [7:0] ctrl8_r      , //  Read Data from Register 8
  output  reg [7:0] ctrl9_r      , //  Read Data from Register 9
  output  reg [7:0] ctrlA_r      , //  Read Data from Register A
  output  reg [7:0] ctrlB_r      , //  Read Data from Register B
  output  reg [7:0] ctrlC_r      , //  Read Data from Register C
  output  reg [7:0] ctrlD_r      , //  Read Data from Register D
  output  reg [7:0] ctrlE_r      , //  Read Data from Register E
  output  reg [7:0] ctrlF_r        //  Read Data from Register F
  );
    
   
// JTAG Registers
   wire  [ 7:0] sciwdata, scirmxdata;
   wire  [17:0] sciaddr;

	reg			reg_write_en;
	reg			jcex_r;
	reg		[17:0]	sciaddr_r;
	reg		[7:0]	sciwdata_r;
	reg			sync_sciwstn;
	reg			sync_sciwstn_r;
	reg		[7:0]		ctrl0_r,ctrl1_r,ctrl2_r,ctrl3_r,ctrl4_r,ctrl5_r,ctrl6_r,ctrl7_r;
	reg		[7:0]		ctrl8_r,ctrl9_r,ctrlA_r,ctrlB_r,ctrlC_r,ctrlD_r,ctrlE_r,ctrlF_r;

// Instatiate ORCAstra Module
   orcastra ORCAstra_inst
   (  .rstn       (1'b1      ),
      .scirmxdata (scirmxdata),
      .sciwdata   (sciwdata  ),
      .sciaddr    (sciaddr   ),
      .sciwstn    (sciwstn   ),
      .scird      (scird     ),
      .jcex       (jcex      )
   );

// JTAG Read Data    
   assign scirmxdata = ((sciaddr == 18'h00000)?( ctrl0_r ):(8'h0)) |
                       ((sciaddr == 18'h00001)?( ctrl1_r ):(8'h0)) |
                       ((sciaddr == 18'h00002)?( ctrl2_r ):(8'h0)) |
                       ((sciaddr == 18'h00003)?( ctrl3_r ):(8'h0)) |
                       ((sciaddr == 18'h00004)?( ctrl4_r ):(8'h0)) |
                       ((sciaddr == 18'h00005)?( ctrl5_r ):(8'h0)) |
                       ((sciaddr == 18'h00006)?( ctrl6_r ):(8'h0)) |
                       ((sciaddr == 18'h00007)?( ctrl7_r ):(8'h0)) |
                       ((sciaddr == 18'h00008)?( ctrl8_r ):(8'h0)) |
                       ((sciaddr == 18'h00009)?( ctrl9_r ):(8'h0)) |
                       ((sciaddr == 18'h0000A)?( ctrlA_r ):(8'h0)) |
                       ((sciaddr == 18'h0000B)?( ctrlB_r ):(8'h0)) |
                       ((sciaddr == 18'h0000C)?( ctrlC_r ):(8'h0)) |
                       ((sciaddr == 18'h0000D)?( ctrlD_r ):(8'h0)) |
                       ((sciaddr == 18'h0000E)?( ctrlE_r ):(8'h0)) |
                       ((sciaddr == 18'h0000F)?( ctrlF_r ):(8'h0));
  


// Write Data Coming from JTAG 
   
   always @(negedge sciwstn)  // catch the data from jtag when sciwstn goes low
	   begin
		   jcex_r	<=	jcex;
		   sciaddr_r	<=	sciaddr;
		   sciwdata_r	<=	sciwdata;
		end
		   
	always @(posedge sys_clk_i)  //make a single clock pulse after sciwstn goes low. use it to move data into registers
			if (~rstn_i)
				begin
					reg_write_en <= 1'b0;
				end
			else 
				begin
					sync_sciwstn 	<= 	sciwstn;   // sync it to system clock
					sync_sciwstn_r 	<= 	sync_sciwstn;
					reg_write_en 	<= 	~sync_sciwstn & sync_sciwstn_r;  // make a single pulse
				end
	   

always @(posedge sys_clk_i)
begin
	if(~rstn_i)
	begin
		ctrl0_r <= 8'b0;
		ctrl1_r <= 8'b0;
		ctrl2_r <= 8'b0;
		ctrl3_r <= 8'b0;
		ctrl4_r <= 8'b0;
		ctrl5_r <= 8'b0;
		ctrl6_r <= 8'b0;
		ctrl7_r <= 8'b0;
		ctrl8_r <= 8'b0;
		ctrl9_r <= 8'b0;
		ctrlA_r <= 8'b0;
		ctrlB_r <= 8'h49;  // format ID = 73 decimal  bit wise 1 channel DDR centered timing
		ctrlC_r <= 8'b0;
		ctrlD_r <= 8'b0;
		ctrlE_r <= 8'h01;	//  Firmware revision 
										// version 01 initial debug
		ctrlF_r <= 8'h00;  

	end
	else
	begin
		if(reg_write_en && jcex_r && (sciaddr_r == 18'h00000))  // write enable
			ctrl0_r <= sciwdata_r;

		if(reg_write_en && jcex_r && (sciaddr_r == 18'h00001))
			ctrl1_r <= sciwdata_r;

		if(done)
			ctrl2_r	<= 8'h00;
			else
				if(reg_write_en && jcex_r && (sciaddr_r == 18'h00002))
					ctrl2_r <= sciwdata_r;

		if(reg_write_en && jcex_r && (sciaddr_r == 18'h00003))
			ctrl3_r <= sciwdata_r;

		if(reg_write_en && jcex_r && (sciaddr_r == 18'h00004))
			ctrl4_r <= sciwdata_r;

		if(reg_write_en && jcex_r && (sciaddr_r == 18'h00005))
			ctrl5_r <= sciwdata_r;

		if(reg_write_en && jcex_r && (sciaddr_r == 18'h00006))
			ctrl6_r <= sciwdata_r;

		if(reg_write_en && jcex_r && (sciaddr_r == 18'h00007))
			ctrl7_r <= sciwdata_r;

		if(reg_write_en && jcex_r && (sciaddr_r == 18'h00008))
			ctrl8_r <= sciwdata_r;

		if(reg_write_en && jcex_r && (sciaddr_r == 18'h00009))
			ctrl9_r <= sciwdata_r;

		if(reg_write_en && jcex_r && (sciaddr_r == 18'h0000A))
			ctrlA_r <= sciwdata_r;

		if(reg_write_en && jcex_r && (sciaddr_r == 18'h0000B))
			ctrlB_r <= sciwdata_r;

		if(reg_write_en && jcex_r && (sciaddr_r == 18'h0000C))
			ctrlC_r <= sciwdata_r;

		if(reg_write_en && jcex_r && (sciaddr_r == 18'h0000D))
			ctrlD_r <= sciwdata_r;

		if(reg_write_en && jcex_r && (sciaddr_r == 18'h0000E))
			ctrlE_r <= sciwdata_r;

		if(reg_write_en && jcex_r && (sciaddr_r == 18'h0000F))
			ctrlF_r <= sciwdata_r;

	end
end
      
endmodule

module jtagconn16
(  .er2_tdo   (er2_tdo   ),
   .jtck      (jtck      ),
   .jtdi      (jtdi      ),
   .jshift    (jshift    ),
   .jupdate   (jupdate   ),
   .jrstn     (jrstn     ),
   .jce2      (jce2      ),
   .ip_enable (ip_enable )
);
   input  er2_tdo   ;
   output jtck      ;
   output jtdi      ;
   output jshift    ;
   output jupdate   ;
   output jrstn     ;
   output jce2      ;
   output ip_enable ;
endmodule
